<?php
/**
 * Plugin Name: Eventist
 * Plugin URI: http://wordpress.oldalmuhely.hu/eventist/eventist.zip
 * Description: Easy event manager for WordPress CMS
 * Version : 0.2
 * Author: Viktor Surek (iceseven)
 * Author URI : http://profiles.wordpress.org/ices3ven/
 * License: GPLv2 or later
**/
/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
*/


/**
 * Regular definitions
**/
define('EVENTIST_URL',plugin_dir_url(__FILE__));//http://.../plugins/eventist/
define('EVENTIST_DIR',plugin_dir_path(__FILE__));//C:\..\plugins\eventist/
define('EVENTIST_DETAILS_LEN',80);//Length of the details in the event list table
define('EVENTIST_OPT_PAGE','eventist_own_pageid');//option name for selected page id
define('EVENTIST_DISP_TYPE','eventist_own_disp_type');//display type option
/**
 * Some files need to be included!
 * var array
**/
$loadit = array(
'eventist_core.php',
'eventist_admin.php',
'eventist_table.php',
'eventist_widget.php',
'eventist_calendar.php'
);
/**
 * Loads necessary include files
 * @param array $filenames - filenames to load (relative to Eventist root)
 * @return bool
**/
function loadIncludes($filenames = array()) {
    $num = 0;
    if(empty($filenames)) return FALSE;
    $full = count($filenames);
    foreach($filenames as $fn) {
        $path = EVENTIST_DIR.$fn;
        if(file_exists($path)) {
            //Include only once, but if it is already included
            //it does not matter, in this case function returns
            //true too!!
            if(@include_once($fn)) {
                $num++;
            }
        }
    }
    
    if($num == $full) return TRUE;
    
    return FALSE;
}

/**
 * Create optional quantity of menu and submenu pages
 * for the plugin admin
**/
function EvCreateAdminMenus() {
    add_menu_page('Eventist Plugin','Eventist Plugin','edit_posts','eventist-main',array('evAd','createMainPage'));
    add_submenu_page('eventist-main','Settings','Settings','edit_posts','eventist-settings',array('evAd','createSettingsPage'));
     add_submenu_page('eventist-main','About Eventist','About','edit_posts','eventist-about',array('evAd','createAboutSubPage'));
}

/**
 * Adding additional css to the admin
**/
function onEvAdmStyle() {
    wp_register_style('ev_adm_style',EVENTIST_URL.'css/admin_style.css');
    wp_enqueue_style('ev_adm_style');
}

/**
 * Call plugin db table creato function if plugin loaded
**/
function onPluginLoad() {
    @require_once('eventist_core.php');
    evCore::checkDB();
}

/**
 * Load the plugin's jQuery properly
**/
function evJq() {
    wp_enqueue_script('jquery');
    wp_enqueue_script('jquery-ui-core');
    wp_print_scripts( 'jquery-ui-datepicker' );
    wp_deregister_style('evUIS');
    wp_register_style('evUIS',EVENTIST_URL.'js/jqui.css');
    wp_enqueue_style('evUIS');
    //Adding plugin's own js
    wp_deregister_script('evAdd');
    wp_register_script('evAdd',EVENTIST_URL.'js/addevent.js');
    wp_enqueue_script('evAdd');
}

/**
 * Load eventist front end CSS
**/
function onFrontLoad() {
    wp_deregister_style('evTB');
    wp_register_style('evTB',EVENTIST_URL.'css/eventist_cstyle.css');
    wp_enqueue_style('evTB');
}

/**
 * Extends the content of the selected page
 * with event display
**/
function pageCreation($content) {
    global $post;
    if($post->ID == get_option(EVENTIST_OPT_PAGE) && evAd::getReqVars('page_id',1)) {
        //
        $mycal = new evCalendar(date("Y"),date("m"));
        $calendar_content = $mycal->displayOnPage();
        $wrt = $content.$calendar_content;
    } else {
        $wrt = $content;
    }
    
    return $wrt;
}

/**
 * Registering and activating eventist widget
**/
function evWidgetStart() {
    register_widget('evWid');
}

add_action('plugins_loaded','onPluginLoad');
add_action('widgets_init','evWidgetStart');

if(loadIncludes($loadit)) {
    
    if(is_admin()) {
        //Admin menu
        add_action('admin_enqueue_scripts','evJq');
        add_action('admin_menu','onEvAdmStyle');
        add_action('admin_menu','EvCreateAdminMenus');
    } else {
        //Display for WordPress front
        add_action('init','onFrontLoad');
        add_filter('the_content','pageCreation');
    }
    
    
    
}


?>