<?php
/**
 * On the Eventist frontend this will show a monthly calendar
 * for the events
**/


class evCalendar {
    
    private $year;
    private $month;
    
    /**
     * Constructor
     * @param int $year - the year of the calendar
     * @param int $month - month of the calendar
    **/
    public function __construct($year,$month) {
        $this->year = $year;
        $this->month = $month;
    }
    
    public function display($type = 1,$pagelink = true) {
        //If no needs to display table, use other display function
        if($type !== 1) {
            $out = $this->listDisplay();
            return $out;
        }
        $sev = $this->collectStartDate();
        $alld = $this->getDates();
        $a = $this->shakeDates($sev,$alld);
        $week = $this->weeklyOrder($a);
        $out = '
        <table border="0" id="eventist-table">
            <thead>
                <tr>
                    <th>Mo</th>
                    <th>Tu</th>
                    <th>We</th>
                    <th>Thu</th>
                    <th>Fri</th>
                    <th>Sat</th>
                    <th>Su</th>
                </tr>
            </thead>';
        $s_brd = 0;
        $e_brd = 6;
        for($i=0;$i<$this->getTableNums();$i++) {
            if($i == $s_brd) {
                $out .= '<tr>';
                $s_brd +=7;
            }
            $out .= '<td>';
            if(!empty($week[$i])) {
                $c_link = '';
                if($res = evCore::getSingleByDate($week[$i]['full'])) {
                    if(!empty($res[0]['title'])) {
                        $c_link = '<a href="?page_id='.get_option(EVENTIST_OPT_PAGE).'&eventistd='.$week[$i]['full'].'" title="'.$res[0]['title'].'">'.$week[$i]['day'].'</a>';
                    } else {
                        $c_link = '<a href="?page_id='.get_option(EVENTIST_OPT_PAGE).'&eventistd='.$week[$i]['full'].'" title="...">'.$week[$i]['day'].'</a>';
                    }
                } else {
                    $c_link = $week[$i]['day'];
                }
                $out .= $c_link;
            } else {
                $out .= '';
            }
            $out .= '</td>';
            if($i == $e_brd) {
                $brd += 7;
                $out .= '</tr>';
            }
        }       
        
        $out .= '</table>
        ';
        if($pagelink) $out .= '<p><a href="?page_id='.get_option(EVENTIST_OPT_PAGE).'">Events page</a></p>';
        
        return $out;
    }
    
    /**
     * Return a list of events selected by given year and month
     * @return string
    **/
    public function listDisplay() {
        $events = evCore::getEventsByMonth($this->month);
        $events = array_reverse($events);
        foreach($events as $ev) {
            $out .= '
            <p>
                <i><strong>'.$ev['start_d'].'</strong></i>:<br/>
                <a href="?page_id='.evAd::getReqVars('page_id',1).'&eventistid='.$ev['id'].'">'.$ev['title'].'</a>
            </p>
            ';
        }
        
        return $out;
    }
    
    /**
     * Return a list of events of a specified day
     * @param array $events - the event array
     * @return string
    **/
    public function listTypeDisplay($events) {
        foreach($events as $ev) {
            $out .= '
            <p>
                <i><strong>'.$ev['start_d'].'</strong></i>:<br/>
                <a href="?page_id='.evAd::getReqVars('page_id',1).'&eventistid='.$ev['id'].'">'.$ev['title'].'</a>
            </p>
            ';
        }
        
        return $out;
    }
    
    /**
     * Return a single event content
     * @param int $id - event ID
     * @return string
    **/
    public function singleTypeDisplay($id) {
        $ev = evCore::getEvent($id);
        if(!empty($ev['end_d']) && $ev['end_d'] != '0000-00-00') {
            $end_d_content = ' - <i>'.$ev['end_d'].'</i>';
        }
        $out = '
        <p>
        <h2><u>'.$ev['title'].'</u></h2>
        <i>'.$ev['start_d'].'</i>'.$end_d_content.'<br/>
        '.$ev['location'].'<br/>
        '.$ev['details'].'
        </p>
        ';
        
        return $out;
    }
    
    /**
     * Create an array with eventist format dates of the current 
     * month
     * @return array
    **/
    private function getDates() {
        $cdate = array();
        for($i=0;$i<32;$i++) {
            $fulldate = mktime(0,0,0,$this->month,$i,$this->year);
            $fulldate = date("Y-m-d",$fulldate);
            if(checkdate($this->month,$i,$this->year)) {
                array_push($cdate,$fulldate);
            }
        }
        
        return $cdate;
    }
    
    /**
     * Collect event start dates of the current month
     * @return array
    **/
    private function collectStartDate() {
        $events = evCore::getEventsByDate(array($this->year,$this->month));
        $edates = array();
        for($i=0;$i<count($events);$i++) {
            if(!in_array($events[$i]['start_d'],$edates)) array_push($edates,$events[$i]['start_d']);
        }
        
        return $edates;
    }
    
    /**
     * Put the dates of the current month and the dates of events into one array
     * @param array $start_dates - all event start dates of current month in an array
     * @param array $all_dates - all dates of the current month
     * @return array
    **/
    private function shakeDates($start_dates,$all_dates) {
        $parray = array();
        for($i=0;$i<count($all_dates);$i++) {
            $parray[$i]['full'] = $all_dates[$i];
            $d = date_parse($all_dates[$i]);
            $d = mktime(0,0,0,$d['month'],$d['day'],$d['year']);
            $d_n = date("w",$d);
            $d = date("d",$d);
            $parray[$i]['day'] = $d;
            $parray[$i]['w'] = $d_n;
        }
        
        return $parray;
    }
    
    /**
     * Order the dates in a weekly orderes array
     * @param array $dates - the full date array
     * @return array
    **/
    private function weeklyOrder($dates) {
        //Start of the month
        $ds = date_parse($dates[0]['full']);
        $ds = mktime(0,0,0,$ds['month'],$ds['day'],$ds['year']);
        $ds = date("w",$ds);

        $wday = array(1,2,3,4,5,6,0);
        for($i=0;$i<count($wday);$i++) {
            if($ds == $wday[$i]) $spos = $i;
        }
        
        $wcurrent = array();
        $db = 0;
        for($i=0;$i<$this->getTableNums();$i++) {
            if($i>=$spos && !empty($dates[$db])) {
                $wcurrent[$i] = $dates[$db];
                $db++;
            } else {
                $wcurrent[$i] = '';
            }
        }
        
        return $wcurrent;
        
    }
    
    /**
     * The rows of the calendar table depends on what day the current
     * month begin. This function returns the nums of rows in current calendar table
     * @
     * @return int
    **/
    private function getTableNums() {
        $ds = date_parse($this->year.'-'.$this->month.'-01');
        $ds = mktime(0,0,0,$ds['month'],$ds['day'],$ds['year']);
        $ds = date("w",$ds);

        $wday = array(1,2,3,4,5,6,0);
        for($i=0;$i<count($wday);$i++) {
            if($ds == $wday[$i]) $spos = $i;
        }
        
        if($wday[$spos] == 6 || $wday[$spos] === 0) return 42;
        
        return 35;
    }
    
    
    /**
     * Showing content on Eventist selected page
     * @return string
    **/
    public function displayOnPage() {
        if(evAd::getReqVars('eventistd',1)) {
            $out = $this->listTypeDisplay(evCore::getAllByDate(evAd::getReqVars('eventistd',1)));
        } else if(evAd::getReqVars('eventistid',1)) {
            $out = $this->singleTypeDisplay(evAd::getReqVars('eventistid',1));
        } else {
            //Eventist page main page
            $dtype = get_option(EVENTIST_DISP_TYPE);
            if($dtype == 1) $dtype = 1;
            if(evAd::getReqVars('eventisty',1) && evAd::getReqVars('eventistm',1)) {
                //Create a new table of events with a given daterange
                $ni = new evCalendar(evAd::getReqVars('eventisty',1),evAd::getReqVars('eventistm',1));
                $out = $ni->createRangeSelect(evAd::getReqVars('eventisty',1),evAd::getReqVars('eventistm',1));
                $out .= $ni->display($dtype,false);
            } else {
                $out = $this->createRangeSelect(date("Y"),date("m"));
                $out .= $this->display($dtype,false);
            }
            
            
        }
        return $out;
    }
    
    /**
     * Create a form for table display, where user can select year and month
     * to display a given calendar table
     * @return string
    **/
    public function createRangeSelect($y,$m) {
        $all_dates = evCore::getAllYears();
        $years = array();
        if(empty($all_dates)) return '';
        foreach($all_dates as $date) {
            $d = evCore::extractEvDate($date['start_d'],"Y");
            if(!in_array($d,$years)) {
                array_push($years,$d);
            }
        }
        $yearselect = '<select name="eventisty" id="eventisty">';
            foreach($years as $year) {
                $chk = '';
                if($y == $year) $chk = ' selected="selected"';
                $yearselect .= '<option value="'.$year.'"'.$chk.'>'.$year;
            }
        $yearselect .= '</select>';
        
        $monthselect = '<select name="eventistm" id="eventistm">';
        $mymonth = array('01','02','03','04','05','06','07','08','09','10','11','12');
            for($i=0;$i<12;$i++) {
                $chk = '';
                if($m == $mymonth[$i]) $chk = ' selected="selected"';
                $monthselect .= '<option value="'.$mymonth[$i].'"'.$chk.'>'.$mymonth[$i];
            }
        $monthselect .= '</select>';
        
        $or = '';
        if($y != date("Y") || $m != date("m")) $or = ' or go to current month:<a href="?page_id='.evAd::getReqVars('page_id',1).'">'.date("Y-m").'</a>';
        
        $form = '
        <p>
        <form method="get">
        <input type="hidden" name="page_id" value="'.evAd::getReqVars('page_id',1).'" />
            Select year and month to display events:<br/>
            Year:&nbsp;'.$yearselect.'&nbsp;
            Month:&nbsp;'.$monthselect.'
            <input type="submit" />&nbsp; '.$or.'
        </form>
        </p>
        ';
        
        return $form;
    }
    
}

?>