<?php
/**
 * Eventist core singelton class to add, save, delete, update and
 * listing events
**/

class evCore {
    /**
     * The database table used by this plugin
     * @var string
    **/
    const TBNAME = 'eventist_own_tb';
    
    /**
     * Get all of the events from the database
     * @return array
     * @todo End this function, with properly made return value
    **/
    public static function getEvents() {
        global $wpdb;
        $sql = 'SELECT * FROM '.self::getTbName();
        $res = $wpdb->get_results($sql,ARRAY_A);
        
        //var_dump($res);
        return $res;
    }
    
    /**
     * Function for shorting events by some aspect
     * @param string $ob - order by
     * @param string $o - order type , ASC or DESC
     * @param array $lim - SQL limit parameters
     * @retun array
    **/
    public static function getEventsShorted($ob,$o,$lim=array()) {
        global $wpdb;
        //Changing thge limit
        $limit = '';
        if(!empty($lim)) {
            $limit = ' LIMIT '.$lim[0];
            if(!empty($lim[1]) && $lim[1] != 0) $limit .= ', '.$lim[1];
        }
        $ql = 'SELECT * FROM '.self::getTbName().' ORDER BY '.$ob.' '.$o.$limit;
        $ql = $wpdb->prepare($ql);
        $res = $wpdb->get_results($ql,ARRAY_A);
        return $res;
    }
    
    /**
     * Get events by data or date range
     * @param array $sr - start range, eg.:array(2012,02) means from 2012.02
     * @param array $er - end range, eg.: array(2013.02) means same as above
     * @return array
    **/
    public static function getEventsByDate($sr,$er=array()) {
        global $wpdb;
        $sdate = mktime(0,0,0,$sr[1],1,$sr[0]);
        $sdate = date("Y-m-d",$sdate);
        $and = '';
        if(!empty($er)) {
            $edate = mktime(0,0,0,$er[1],1,$er[0]);
            $edate = date("Y-m-d",$edate);
            if(!$edate=self::extractEvDate($edate,"Y-m-d")) return FALSE;
            $and = ' AND end_d<=\''.$edate.'\'';
        }
        if($sdate = self::extractEvDate($sdate,"Y-m-d")) {
            $sql = 'SELECT * FROM '.evCore::getTbName().' WHERE start_d >=\''.$sdate.'\''.$and;
            if(!$sql = $wpdb->prepare($sql)) return FALSE;
            $res = $wpdb->get_results($sql,ARRAY_A);
            if($res) {
                return $res;
            }
            return FALSE;
        }
        
        return FALSE;
    }
    
    /**
     * Get events by specified month
     * @param string $month - 01-12 format month
     * @return array
    **/
    public static function getEventsByMonth($m) {
    
        $ev = self::getEvents();
        if(empty($ev)) return array();
        $ev_ret = array();
        foreach($ev as $e) {
            $d = date_parse($e['start_d']);
            if($d['month'] == $m) {
                array_push($ev_ret,$e);
            }
        }
        
        return $ev_ret;
    }
    
    /**
     * Get one event from db by id
     * @param int $id - one event id
     * @return array (associative array)
    **/
    public static function getEvent($id) {
        global $wpdb;
        $query = 'SELECT * FROM '.self::getTbName().' WHERE id='.$id;
        $query = $wpdb->prepare($query);
        
        $res = $wpdb->get_results($query,ARRAY_A);
        
        return $res[0];
    }
    
    /**
     * Get the first event of given date from the DB
     * @param string $date - the given date
     * @return mixed
    **/
    public static function getSingleByDate($date) {
        global $wpdb;
        if($date = self::extractEvDate($date,"Y-m-d")) {
            $sql = 'SELECT * FROM '.self::getTbName().' WHERE start_d=\''.$date.'\' LIMIT 1';
            if($wpdb->prepare($sql)) {
                $res = $wpdb->get_results($sql,ARRAY_A);
                if($res) {
                    return $res;
                }
                
                return FALSE;
            }
            
            return FALSE;
        }
        
        return FALSE;
    }
    
    /**
     * Get all events of a specified full date (means all events of a day)
     * @param string $date - the given date
     * @return array
    **/
    public static function getAllByDate($date) {
        global $wpdb;
        if($date = self::extractEvDate($date,"Y-m-d")) {
            $sql = 'SELECT * FROM '.self::getTbName().' WHERE start_d=\''.$date.'\'';
            if($wpdb->prepare($sql)) {
                $res = $wpdb->get_results($sql,ARRAY_A);
                if($res) {
                    return $res;
                }
                
                return FALSE;
            }
            
            return FALSE;
        }
        
        return FALSE;
    }
    
    /**
     * Save a newly created event or update an existing one
     * @param array $event - assoc array with all event information
     * @return bool
     * @todo with a second parameter, a reference parameter send error
     *  message to external world
     * @todo Now we can add a start date earlyer then current date, is it right?
    **/
    public static function saveEvent($event) {
        global $wpdb;
        //If we got an ID too, it will be an update
        $store = array();
        //Get the curent user
        $store['evuser'] = wp_get_current_user()->ID;
        //Get current date
        $store['evdate'] = date("Y-m-d H:i:s");
        //If there is no start date we return false
        if(!isset($event['start_d'])) return FALSE;
        $store['start_d'] = self::extractEvDate($event['start_d'],"Y-m-d");
        //If we got no proper format
        if(!$store['start_d']) return false;
        //See, if we have multiday event or not
        if(isset($event['notsameday']) && $event['notsameday'] === '1') {
            $store['end_d'] = self::extractEvDate($event['end_d'],"Y-m-d");
            if(!$store['end_d']) $store['end_d'] = $store['start_d'];
            if($store['start_d'] > $store['end_d']) $store['end_d'] = $store['start_d'];
        } else {
            //in every other cases end date is NULL or same day as startday
            $store['end_d'] = '';
        }
        //Time
        if(!isset($event['time'])) { $store['time'] = ''; } else { $store['time'] = $wpdb->escape($event['time']); }
        //Title
        if(!isset($event['title'])) {$store['title'] = ''; } else { $store['title'] = $wpdb->escape($event['title']); }
        //Location
        if(!isset($event['location'])) { $store['location'] = ''; } else { $store['location'] = $wpdb->escape($event['location']); }
        //Details
        if(!isset($event['details'])) { $store['details'] = ''; } else { $store['details'] = $wpdb->escape($event['details']); }
        if(isset($event['id']) && !empty($event['id'])) {
            //Update an existing one
            $res = $wpdb->update(self::getTbName(),$store,array('id'=>$event['id']));
            if($res === false) return FALSE;
            return TRUE;
        } else {
            //New
            $res = $wpdb->insert(self::getTbName(),$store);
            if($res == false) return FALSE;
            return TRUE;
        }
    }
    
    /**
     * Copies an existing event to the database, with a new ID
     * @param int $id - event id
     * @return bool
    **/
    public static function copyEvent($id) {
        $event = self::getEvent($id);
        if(empty($event)) return array();
        $store = array();
        foreach($event as $k=>$v) {
            if($k != 'id') {
                $store[$k] = $v;
            }
        }
        if(!empty($store)) {
            if(self::saveEvent($store)) return TRUE;
            
            return FALSE;
        } else {
            return FALSE;
        }
    }
    
    /**
     * If there is no Db for the plugin, this will create one!
    **/
    public static function checkDB() {
       $db_sql = 'CREATE TABLE '.self::getTbName().' (
	   id int(12) NOT NULL AUTO_INCREMENT,
	   evuser bigint(20) NOT NULL,
	   evdate datetime NOT NULL DEFAULT "0000-00-00 00:00:00",
	   start_d date NOT NULL DEFAULT "0000-00-00",
	   end_d date DEFAULT NULL,
	   time text,
	   title text NOT NULL,
	   location text,
	   details text,
	   history text,
	   PRIMARY KEY  (id) )
	   DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;';
       //If not included, we get it!
       //This contains dbDelta to create the db table
       require_once( ABSPATH.'wp-admin/includes/upgrade.php' );
       dbDelta($db_sql);
        
    }
    
    /**
     * Returns the full db table name with prefix
     * @return string
    **/
    public static function getTbName() {
        global $wpdb;
        return $wpdb->prefix.self::TBNAME;
    }
    
    /**
     * Delet an event by given event id
     * @param int $id - event id or ids(maye comma separated)
     * @return void
     * @todo PROPER RETURN VALUE NEEDED!!
    **/
    public static function delEvent($id) {
        global $wpdb;
        $query = 'DELETE FROM '.self::getTbName().' WHERE id IN ('.$id.')';
        $res = $wpdb->query($wpdb->prepare($query));
        
    }
    
    /**
     * Make a usable dateformat from a string date if possible
     * @param string $datestring - string date
     * @param string $ret_format - return format of the date (eg.:"Y-m-d")
     * @return string (formatted date by a given timestamp)
    **/
	public static function extractEvDate( $datestring, $ret_format ) {
		//Parsing the simple string date into a 
        //usable date format assoc array
        $date_array = date_parse($datestring);
        //If some errors occured when parsing return FALSE
		if( !empty($date_array['errors'])) {
			return false;
		}
        //If not valid gregorian date returns false
		if( false === checkdate($date_array['month'], $date_array['day'], $date_array['year'])) {
			return false;
		}
        //Make timestamp from the given data
		$timestamp = mktime(0, 0, 0, $date_array['month'], $date_array['day'], $date_array['year']);
		//Return usable dateformat 
		return date($ret_format,$timestamp);
	}
    
    /**
     * Get first and last date year from the eventist's table
     * @param string $dr - f or l depends on first or last year requested
     * @return string
    **/
    public static function getDbYears($dr) {
        global $wpdb;
        if($dr == 'f') {
            //When first date
            $sd = 'start_d';
            $ord = 'ASC';
        } elseif($dr == 'l') {
            //When last date
            $sd = 'end_d';
            $ord = 'DESC';
            
        }
        $query = 'SELECT DISTINCT '.$sd.' FROM '.self::getTbName().' WHERE '.$sd.' != "0000-00-00" ORDER BY '.$sd.' '.$ord.' LIMIT 1';
        $query = $wpdb->prepare($query);
        $res = $wpdb->get_results($query,ARRAY_A);
        
        if(!empty($res)) {
            return self::extractEvDate($res[0][$sd],"Y");
        } else {
            return date("Y");
        }
        
        
    }
    
    /**
     * Get all years what has any events in it (depends on start date!
     * ,so an event begins 2013.12.31 and ends in 2014, will only adds 2013 to the years)
     * @return array
    **/
    public static function getAllYears() {
        global $wpdb;
        $sd = 'start_d';
        $ord = 'ASC';
        $query = 'SELECT DISTINCT '.$sd.' FROM '.self::getTbName().' WHERE '.$sd.' != "0000-00-00" ORDER BY '.$sd.' '.$ord;
        if($wpdb->prepare($query)) {
            if($res = $wpdb->get_results($query,ARRAY_A)) {
                return $res;
            }
            
            return FALSE;
        }
        
        return FALSE;
    }
    
    /**
     * Get the number of events in DB
     * @return int
    **/
    public static function getNumDb() {
        $res = self::getEvents();
        
        return count($res);
    }
    
    /**
     * Decides wheter the given event is multiday event or not
     * @param int $id - the event id
     * @return bool
    **/
    public function isMultiday($id) {
        $ev = self::getEvent($id);
        $sd = $ev['start_d'];
        $ed = $ev['end_d'];
        if(empty($ed) || $ed == '0000-00-00') return FALSE;
        if($sd >= $ed) return FALSE;
        
        return TRUE;
    }
    
    
}

?>