<?php
/**
 * WP_List_Table implementation
**/
if(!class_exists('WP_List_Table')) {
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class evTable extends WP_List_Table {
    
    public $disp,$srt,$hd;
    private $per_page_num;
    
    /**
     * Own List Table class constructor, where we could add
     * array of data to be displayed
     * @param array $disp - keywords of displaying elements
     * @param array $srt - shortable keys
     * @param array $hd - hidden keys
    **/
    public function __construct($disp = array(),$srt = array(),$hd=array()) {
        global $status,$page;
        $this->disp = $disp;
        $this->srt = $srt;
        $this->hd = $hd;
        $this->per_page_num = 10;
        parent::__construct(array(
            'singular' => 'event',
            'plural' => 'events',
            'ajax' => false
        ));
    }
    
    /**
     * Returns az associative array with displaying table elements with slugs and 
     * diaplaying titles
     * These will be the $this->disp array elements
     * @return array
    **/
    public function get_columns() {
        $dsp['cb'] = '<input type="checkbox" />';
        foreach($this->disp as $key=>$value) {
            $dsp[$key] = $value;
        }
        $this->disp = $dsp;
        
        return $this->disp;
    }
    
    /**
     * This function will prepare data for displaying, collect shiortable, hidden and
     * displaying elements. By the way here could be done the work with
     * shorting incoming DB data, so there is the place to call some
     * shorting function
    **/
    public function prepare_items() {
        $columns = $this->get_columns();
        $per_page = $this->per_page_num;
        $current_page = $this->get_pagenum();
        $total_items = count($this->orderList());
        $sql_start = ($current_page-1)*$per_page;
        $this->set_pagination_args( array(
            'total_items' => $total_items,
            'per_page' => $per_page 
        ) );
        $shortable = $this->get_shortable_columns();
        $hidden = $this->hd;
        $this->_column_headers = array($columns,$hidden,$shortable);
        $this->process_bulk_action();
        $this->items = $this->orderList(array($sql_start,$per_page));
    }
    
    /**
     * Default column format. Now it is just the column title
    **/
    public function column_default( $item, $column_name ) {
        return $item[$column_name];
    }
    
    /**
     * From the data of $this->srt array this function make
     * a shortable column collection, extending the first one, with 
     * necessary additional information
    **/
    public function get_shortable_columns() {
        $sc = array();
        foreach($this->srt as $s) {
            $sc[$s] = array($s,false); 
        }
        
        return $sc;
   }
   
   /**
    * Order the events by the parameters it gets from table list, but deafult
    * case is: events shorted by event added date and descends, so the most recent 
    * will be at the top
    * @param array $lim - SQL limit parameters
    * @return array 
   **/
   public function orderList($lim = array()) {
        $orderby = (!empty($_GET['orderby'])) ? $_GET['orderby'] : 'evdate';
        $order = (!empty($_GET['order'])) ? $_GET['order'] : 'desc';
        $res = evCore::getEventsShorted($orderby,$order,$lim);
        
        return $res;
   }
   
   /**
    * Specific function to display title column and the additional
    * links with it for deleting and editing 
    * @return string
   **/
   public function column_title($item) {
   
     $actions = array(
        'edit' => sprintf('<a href="?page=%s&action=%s&id=%s">Edit</a>',$_REQUEST['page'],'edit',$item['id']),
        'copy' => sprintf('<a href="?page=%s&action=%s&id=%s">Copy</a>',$_REQUEST['page'],'cpy',$item['id']),
        'delete' => sprintf('<a href="?page=%s&action=%s&id[]=%s">Delete</a>',$_REQUEST['page'],'delete',$item['id']),
     );
     
    return sprintf('%1$s %2$s', $item['title'], $this->row_actions($actions) );     
        
   }
   
   /**
    * Specific function to display the name of a user the event created by
    * @return string
   **/
   public function column_evuser($item) {
        
        $name = get_userdata($item['evuser']);
        
        return $name->user_login;
            
   }
   
   /**
    * Specific function to display event details
    * @return string
   **/
   public function column_details($item) {
        $desp = $this->truncate(EVENTIST_DETAILS_LEN,$item['details']);
        
        return $desp;
   }
   
   /**
    * End date will be displayed only when there is real end date
    * in other cases the start date'll be return
    * @return string
   **/
   public function column_end_d($item) {
        
        if($item['end_d'] == '0000-00-00') {
            return $item['start_d'];
        }
        
        return $item['end_d'];
        
   }
   
   /**
    * Create a proper time format for event cration date
    * @param string $d - the date
    * @return string
   **/
   private function format_date($d) {
		$timestamp = strtotime($d);
		$time_diff = time() - $timestamp;
		if( $time_diff >= 0 && $time_diff < 24*60*60 ) {
			$date = sprintf( __( '%s ago' ), human_time_diff( $timestamp ) );
		}
		else {
			$date = mysql2date(__( 'Y/m/d' ), $d);
		}
		$datetime = mysql2date(__( 'Y/m/d g:i:s A' ), $d);
		return '<abbr title="'.$datetime.'">'.$date.'</abbr>';
	}
    
    /**
     * Format the date displayed in column Date Created
     * If it is not older then a day the full date won't be displayed
     * @return string
    **/
    public function column_evdate($item) {
        return $this->format_date($item['evdate']);
    }
    
    /**
     * For bulk actions, attach checkbox on the start of every row
     * @return string
    **/
    public function column_cb($item) {
        return sprintf('<input type="checkbox" name="id[]" value="%1$s" />',$item['id']);
    }
    
    /**
     * Adding group actions
    **/
    function get_bulk_actions() {
        $actions = array(
            'delete' => 'Delete'
        );
        
        return $actions;
    }
    
    function process_bulk_action() {
        
        //Detect when a bulk action is being triggered...
        if( 'delete'===$this->current_action() ) {
            wp_die('Items deleted (or they would be if we had items to delete)!');
        }
        
    }
    
    /**
     * Truncate a HTML formatted text
     * @param int $max_length - the max length of the truncated text
     * @param string $text - the text to shorten
     * @return string
     * @author Soren Lovborg
    **/
    private static function truncate( $max_length, $text ) {
		$printedLength = 0;
		$position = 0;
		$tags = array();
		$out = '';
		while ($printedLength < $max_length && preg_match('{</?([a-z]+)[^>]*>|&#?[a-zA-Z0-9]+;}', $text, $match, PREG_OFFSET_CAPTURE, $position)) {
			list($tag, $tagPosition) = $match[0];
			// Print text leading up to the tag.
			$str = substr($text, $position, $tagPosition - $position);
			if ($printedLength + strlen($str) > $max_length) {
				$out .= substr($str, 0, $max_length - $printedLength);
				$printedLength = $max_length;
				break;
			}
			$out .= $str;
			$printedLength += strlen($str);
			if ($tag[0] == '&') {
				// Handle the entity.
				$out .= $tag;
				$printedLength++;
			}
			else {
				// Handle the tag.
				$tagName = $match[1][0];
				if ($tag[1] == '/')
				{
					// This is a closing tag.
					$openingTag = array_pop($tags);
					assert($openingTag == $tagName); // check that tags are properly nested.
					$out .= $tag;
				}
				else if ($tag[strlen($tag) - 2] == '/') {
					// Self-closing tag.
					$out .= $tag;
				}
				else {
					// Opening tag.
					$out .= $tag;
					$tags[] = $tagName;
				}
			}
			// Continue after the tag.
			$position = $tagPosition + strlen($tag);
		}
		// Print any remaining text.
		if ($printedLength < $max_length && $position < strlen($text)) {
			$out .= substr($text, $position, $max_length - $printedLength);
		}
		if ($max_length < strlen($text)) {
			$out .= "...";
		}
		// Close any open tags.
		while (!empty($tags)) {
			$out .= "</" . array_pop($tags) . ">";
		}
		return $out;
	}
   
    
    
}


?>